@@ -1,5 +1,4 @@
-//! Crate to `#include` C++ headers in your Rust code, and generate
-//! idiomatic bindings using `cxx`. See [include_cpp] for details.
+#![doc = include_str!("../README.md")]
 
 // Copyright 2020 Google LLC
 //
@@ -21,6 +20,8 @@
 // do anything - all the magic is handled entirely by
 // autocxx_macro::include_cpp_impl.
 
+pub mod subclass;
+
 #[allow(unused_imports)] // doc cross-reference only
 use autocxx_engine::IncludeCppEngine;
 
@@ -62,7 +63,7 @@ use autocxx_engine::IncludeCppEngine;
 ///
 /// # User manual - introduction
 ///
-/// [include_cpp] tries to make it possible to include C++ headers and use declared functions
+/// [`include_cpp`] tries to make it possible to include C++ headers and use declared functions
 /// and types as-is. The resulting bindings use wrappers for C++ STL types from the [cxx]
 /// crate such as [cxx::UniquePtr] or [cxx::CxxString].
 ///
@@ -71,7 +72,7 @@ use autocxx_engine::IncludeCppEngine;
 /// * This manual will describe how to include `autocxx` in your build process.
 /// * `autocxx` chooses to generate Rust bindings for C++ APIs in particular ways,
 ///   over which you have _some_ control. The manual discusses what and how.
-/// * The combination of `autocxx` and [cxx] are not perfect. There are some STL
+/// * The combination of `autocxx` and [`cxx`] are not perfect. There are some STL
 ///   types and some fundamental C++ features which are not yet supported. Where that occurs,
 ///   you may need to create some manual bindings or otherwise workaround deficiencies.
 ///   This manual tells you how to spot such circumstances and work around them.
@@ -150,14 +151,14 @@ use autocxx_engine::IncludeCppEngine;
 ///     rsb --> l
 /// ```
 ///
-/// # The `include_cpp` macro
+/// # The [`include_cpp`] macro
 ///
 /// Within the braces of the `include_cpp!{...}` macro, you should provide
 /// a list of at least the following:
 ///
 /// * `#include "cpp_header.h"`: a header filename to parse and include
 /// * `generate!("type_or_function_name")`: a type or function name whose declaration
-///   should be made available to C++.
+///   should be made available to C++. (See the section on Allowlisting, below).
 /// * Optionally, `safety!(unsafe)` - see discussion of [`safety`].
 ///
 /// Other directives are possible as documented in this crate.
@@ -169,9 +170,11 @@ use autocxx_engine::IncludeCppEngine;
 /// # Did it work? How do I deal with failure?
 ///
 /// Once you've achieved a successful build, you might wonder how to know what
-/// bindings have been generated. `cargo expand` will show you. In the (near) future,
-/// it's hoped that `rust-analyzer` will gain support for expanding procedural
-/// macros and you'll be able to see the bindings from Rust IDEs.
+/// bindings have been generated. `cargo expand` will show you. Alternatively,
+/// you can get autocompletion within an IDE supported by Rust analyzer. You'll
+/// need to enable _both_:
+/// * Rust-analyzer: Proc Macro: Enable
+/// * Rust-analyzer: Experimental: Proc Attr Macros
 ///
 /// Either way, you'll find (for sure!) that `autocxx` hasn't been able to generate
 /// bindings for all your C++ APIs. This may manifest as a hard failure or a soft
@@ -191,6 +194,24 @@ use autocxx_engine::IncludeCppEngine;
 ///   but not `autocxx` (for example, at the time of writing `std::array`). See
 ///   the later section on 'combinining automatic and manual bindings'.
 ///
+/// # Allowlisting
+///
+/// How do you inform autocxx which bindings to generate? There are three
+/// strategies:
+///
+/// * *Recommended*: provide various [`generate`] directives in the
+///   [`include_cpp`] macro. This can specify functions or types.
+/// * *Not recommended*: in your `build.rs`, call `Builder::auto_allowlist`.
+///   This will attempt to spot _uses_ of FFI bindings anywhere in your Rust code
+///   and build the allowlist that way. This is experimental and has known limitations.
+/// * *Strongly not recommended*: use [`generate_all`]. This will attempt to
+///   generate Rust bindings for _any_ C++ type or function discovered in the
+///   header files. This is generally a disaster if you're including any
+///   remotely complex header file: we'll try to generate bindings for all sorts
+///   of STL types. This will be slow, and some may well cause problems.
+///   Effectively this is just a debug option to discover such problems. Don't
+///   use it!
+///
 /// # The generated bindings
 ///
 /// ## Pointers, references, and so-forth
@@ -223,7 +244,7 @@ use autocxx_engine::IncludeCppEngine;
 ///
 /// There's not a significant ergonomic problem from the use of [`cxx::UniquePtr`].
 /// The main negative of the automatic boxing into [`cxx::UniquePtr`] is performance:
-/// specifiaclly, the need to
+/// specifically, the need to
 /// allocate heap cells on the C++ side and move data into and out of them.
 /// You don't want to be doing this inside a tight loop (but if you're calling
 /// across the C++/Rust boundary in a tight loop, perhaps reconsider that boundary
@@ -273,6 +294,11 @@ use autocxx_engine::IncludeCppEngine;
 /// (See also the discussion of [`safety`] - if you haven't specified
 /// an unsafety policy, _all_ C++ APIs require `unsafe` so the discussion is moot.)
 ///
+/// If you're given a C++ object by pointer, and you want to interact with it,
+/// you'll need to figure out the guarantees attached to the C++ object - most
+/// notably its lifetime. To see some of the decision making process involved
+/// see the [Steam example](https://github.com/google/autocxx/tree/main/examples/steam-mini/src/main.rs).
+///
 /// ### [`cxx::UniquePtr`]s
 ///
 /// We use [`cxx::UniquePtr`] in completely the normal way, but there are a few
@@ -328,14 +354,12 @@ use autocxx_engine::IncludeCppEngine;
 ///
 /// If you need to create a blank `UniquePtr<CxxString>` in Rust, such that
 /// (for example) you can pass its mutable reference or pointer into some
-/// pre-existing C++ API, there's currently no built in support for that.
-/// You should add an extra C++ API:
+/// pre-existing C++ API, call `ffi::make_string("")` which will return
+/// a blank `UniquePtr<CxxString>`.
 ///
-/// ```cpp
-/// std::string make_blank_string() { return std::string(); }
-/// ```
-///
-/// and then use [`generate`] to make bindings for that.
+/// Don't attempt to use [cxx::let_cpp_string] which will allocate the
+/// string on the stack, and is generally incompatible with the
+/// [cxx::UniquePtr]-based approaches we use here.
 ///
 /// ## Preprocessor symbols
 ///
@@ -382,6 +406,10 @@ use autocxx_engine::IncludeCppEngine;
 ///
 /// `autocxx` doesn't yet support default paramters.
 ///
+/// It's fairly likely we'll change the model here in the future, such that
+/// we can pass tuples of different parameter types into a single function
+/// implementation.
+///
 /// ## Forward declarations
 ///
 /// A type which is incomplete in the C++ headers (i.e. represented only by a forward
@@ -408,6 +436,14 @@ use autocxx_engine::IncludeCppEngine;
 /// you can expect serious runtime explosions. The underlying [cxx] crate has
 /// exception support, so it would be possible to add them.
 ///
+/// # Subclasses
+///
+/// There is limited and experimental support for creating Rust subclasses of
+/// C++ classes. (Yes, even more experimental than all the rest of this!)
+/// See [`subclass::CppSubclass`] for information about how you do this.
+/// This is useful primarily if you want to listen out for messages broadcast
+/// using the C++ observer/listener pattern.
+///
 /// # Mixing manual and automated bindings
 ///
 /// `autocxx` uses [cxx] underneath, and its build process will happily spot and
@@ -446,6 +482,8 @@ use autocxx_engine::IncludeCppEngine;
 ///
 /// * [Demo](https://github.com/google/autocxx/tree/main/demo) - simplest possible demo
 /// * [S2 example](https://github.com/google/autocxx/tree/main/examples/s2) - example using S2 geometry library
+/// * [Steam example](https://github.com/google/autocxx/tree/main/examples/steam-mini) - example using (something like) the Steam client library
+/// * [Subclass example](https://github.com/google/autocxx/tree/main/examples/subclass) - example using subclasses
 /// * [Integration tests](https://github.com/google/autocxx/blob/main/engine/src/integration_tests.rs)
 ///   - hundreds of small snippets
 ///
@@ -600,6 +638,27 @@ macro_rules! exclude_impls {
     ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
 }
 
+/// Declare that a given type is a Rust type. These may be used within
+/// references in the signatures of C++ functions, for instance.
+/// This will contribute to an `extern "Rust"` section of the generated
+/// `cxx` bindings.
+#[macro_export]
+macro_rules! rust_type {
+    ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
+}
+
+/// Declare a Rust subclass of a C++ class.
+/// A directive to be included inside
+/// [include_cpp] - see [include_cpp] for general information.
+/// Use this in conjunction with [`subclass::is_subclass`].
+/// See [`subclass::CppSubclass`] for information about the
+/// multiple steps you need to take to be able to make Rust
+/// subclasses of a C++ class.
+#[macro_export]
+macro_rules! subclass {
+    ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
+}
+
 #[doc(hidden)]
 #[macro_export]
 macro_rules! usage {
@@ -620,7 +679,7 @@ pub use autocxx_macro::include_cpp_impl;
 macro_rules! ctype_wrapper {
     ($r:ident, $c:expr, $d:expr) => {
         #[doc=$d]
-        #[derive(Debug, Eq, Clone, PartialEq, Hash)]
+        #[derive(Debug, Eq, Copy, Clone, PartialEq, Hash)]
         #[allow(non_camel_case_types)]
         #[repr(transparent)]
         pub struct $r(pub ::std::os::raw::$r);
